
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
//	_xinfreq = 20_000_000,
	_xtlfreq = 20_000_000,
	_clkfreq = 4_000_000,
	DOWNLOAD_BAUD = 230_400,
	DEBUG_BAUD = DOWNLOAD_BAUD,
	
	PINS = 20|7<<6,
	GROUP = 1,
};



static uint32_t  pinout8_dma( unsigned pgroup, uint8_t *haddr, unsigned len )
{
	uint32_t  xfrq = 0x8000_0000;    // sysclock/1 data rate, top speed!
	uint32_t  m_tx = X_RFBYTE_8P_1DAC8 | pgroup<<20 | X_PINS_ON | len;  // byte wide transfers from hubRAM, via the FIFO
	uint32_t  ticks = _cnt();

__asm volatile {    // "volatile" enforces Fcache use, which is needed to free up the FIFO
		rdfast	#0, haddr    // setup the FIFO to read from hubRAM
		setxfrq	xfrq    // set the transfer rate
		xinit	m_tx, #0    // do it!
		waitxfi   // wait for transfer to complete before returning back to hubexec, hubexec uses the FIFO
}

	return  _cnt() - ticks;
}


static uint32_t  pinout8_grp2( unsigned pgroup, uint8_t *haddr, unsigned len )
{
	uint32_t  val, ticks = _cnt();

__asm volatile {
.loop
		rdbyte	val, haddr    // access hubRAM, intervals of 24 ticks
		add	haddr, #1
		altsb	pgroup, #0x1fc  // OUTA cog register
		setbyte	val
		djnz	len, #.loop
}

	return  _cnt() - ticks;
}


static uint32_t  pinout8_grp1( unsigned pgroup, uint8_t *haddr, unsigned len )
{
	uint32_t  val, ticks = _cnt();

__asm {
.loop
		rdbyte	val, haddr    // access hubRAM, intervals of 24 ticks
		add	haddr, #1
		altsb	pgroup, #0x1fc  // OUTA cog register
		setbyte	val
		djnz	len, #.loop
}

	return  _cnt() - ticks;
}


static uint32_t  pinout8_grp( unsigned pgroup, uint8_t *haddr, unsigned len )
{
	uint8_t  val;
	uint32_t  i, ticks = _cnt();

	for( i=0; i<len; i++ )  {
		val = *(haddr++);
__asm {
//__asm const {
		altsb	pgroup, #0x1fc  // OUTA cog register
		setbyte	val
}
	}

	return  _cnt() - ticks;
}


static uint32_t  pinout8_bb( unsigned pins, uint8_t *haddr, unsigned len )
{
	uint8_t  val;
	uint32_t  i, ticks = _cnt();

	for( i=0; i<len; i++ )  {
		val = *(haddr++);
		_pinwrite( pins, val );
	}

	return  _cnt() - ticks;
}


static uint32_t  filldat( uint8_t *haddr, unsigned len )
{
	uint32_t  i, ticks = _cnt();

	for( i=0; i<len; i++ )  {
		*(haddr++) = (uint8_t)(len - i);
	}

	return  _cnt() - ticks;
}



void  main( void )
{
	uint8_t  buff[1000];

	_waitms( 200 );
	printf( "\n   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );

	printf( "\narray size is %d bytes.  filldat() in %d ticks.\n", sizeof(buff), filldat( buff, sizeof(buff) ) );
	for(;;) // ever
	{
		_pinl( PINS );
		printf( "pinout8_bb() in %d ticks.\n", pinout8_bb( PINS, buff, sizeof(buff) ) );
		_pinf( PINS );

		_pinl( GROUP<<3|7<<6 );
		printf( "pinout8_grp() in %d ticks.\n", pinout8_grp( GROUP, buff, sizeof(buff) ) );
		printf( "pinout8_grp1() in %d ticks.\n", pinout8_grp1( GROUP, buff, sizeof(buff) ) );
		printf( "pinout8_grp2() in %d ticks.\n", pinout8_grp2( GROUP, buff, sizeof(buff) ) );
		printf( "pinout8_dma() in %d ticks.\n\n", pinout8_dma( GROUP, buff, sizeof(buff) ) );
		_pinf( GROUP<<3|7<<6 );

		_waitms( 800 );
	};
}
